#! /usr/bin/env python
# -*- coding: utf-8 -*-
####################
# Indigo Tapo plugin
# Copyright (c) 2022 Alan Carter. All rights reserved.
#

################################################################################
# Imports
################################################################################

import sys
import os
import json
import time
import datetime
import re
from copy import copy
import ipaddress
from tapo_plug import tapoPlugApi


################################################################################
# Globals
################################################################################

def updateVar(name, value, folder=0):

    if name not in indigo.variables:
        indigo.variable.create(name, value=value, folder=folder)
    else:
        indigo.variable.updateValue(name, value)

################################################################################
class Plugin(indigo.PluginBase):

    ########################################
    # Class properties
    ########################################

    def __init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs):
     
        indigo.PluginBase.__init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs)
        self.userName = pluginPrefs.get("userName", "")
        self.userPass = pluginPrefs.get("userPass", "")
        self.showEnergyInNotes = pluginPrefs.get("energyNotes", False)
        self.logComms = pluginPrefs.get("logComms", False)
    
    ########################################
    def __del__(self):
    
        indigo.PluginBase.__del__(self)
        
    ########################################
    def startup(self):
    
        indigo.server.log("Starting TP-Link Tapo plugin")


    def deviceStartComm(self, dev):
    
        dev.stateListOrDisplayStateIdChanged()
        return


    def shutdown(self):
    
        indigo.server.log("Stopping TP-Link Tapo plugin")
        pass
  
    ########################################
    
    def runConcurrentThread(self):
    
        indigo.server.log("Starting TP-Link Tapo monitoring thread")
        while True:
            self.getReadings()
            self.sleep(10)
                        


    ########################################
    # Called when user prefs are changed
    
    def closedPrefsConfigUi(self, valuesDict, userCancelled):
    
        # If the user saved preferences, update logging parameters
        if userCancelled == False:
            energyNotes = self.showEnergyInNotes
            self.showEnergyInNotes = valuesDict.get("energyNotes", False)
            if self.showEnergyInNotes != energyNotes:
                if self.showEnergyInNotes:
                    indigo.server.log("Energy will be shown in Notes column")
                else:
                    indigo.server.log("Energy will not be shown in Notes column")
            logComms = self.logComms
            self.logComms = valuesDict.get("logComms", False)
            if self.logComms != logComms:
                if self.logComms:
                    indigo.server.log("Comms logging is on")
                else:
                    indigo.server.log("Comms logging is off")


        
    ########################################
    # Tapo specific functions
    ########################################

    def getReadings(self):
    
        for dev in indigo.devices.iter("self"):
            device = {
                    "tapoIp": dev.address,
                    "tapoEmail": self.userName,
                    "tapoPassword": self.userPass
            }
            # Need a try statement here because device does not reply
            #   while being accessed via the Tapo app
            try:
                response = tapoPlugApi.getDeviceInfo(device)
                if self.logComms:
                    indigo.server.log("Device info for %s" % dev.name)
                    indigo.server.log(str(response))
                jresponse = json.loads(response)
                onState = jresponse["result"]["device_on"]
                if onState:
                    dev.updateStateOnServer(key="onOffState", value=True, clearErrorState=True)
                    dev.updateStateImageOnServer(indigo.kStateImageSel.PowerOn)
                else:
                    dev.updateStateOnServer(key="onOffState", value=False, clearErrorState=True)
                    dev.updateStateImageOnServer(indigo.kStateImageSel.PowerOff)
                dev.updateStateOnServer(key="firmware", value=jresponse["result"]["fw_ver"])
                if dev.deviceTypeId == "tapoBulb":
                    bLevel = int(jresponse["result"]["brightness"])
                    dev.updateStateOnServer(key="brightnessLevel", value=bLevel, uiValue=str(bLevel)+"%")
                    response = tapoPlugApi.getDeviceUsage(device)
                    if self.logComms:
                        indigo.server.log("Device usage for %s" % dev.name)
                        indigo.server.log(str(response))
                    jresponse = json.loads(response)
                    mT3 =  jresponse["result"]["time_usage"]["today"]
                    mW3 =  jresponse["result"]["time_usage"]["past7"]
                    mM3 =  jresponse["result"]["time_usage"]["past30"]
                    eT3 =  jresponse["result"]["power_usage"]["today"]
                    eW3 =  jresponse["result"]["power_usage"]["past7"]
                    eM3 =  jresponse["result"]["power_usage"]["past30"] 
                    dev.updateStateOnServer(key="WhToday", value=eT3, uiValue=str(eT3)+" Wh") 
                    dev.updateStateOnServer(key="WhPast7Days", value=eW3, uiValue=str(eW3)+" Wh")
                    dev.updateStateOnServer(key="WhPast30Days", value=eM3, uiValue=str(eM3)+" Wh") 
                    dev.updateStateOnServer(key="minsToday", value=mT3, uiValue=str(mT3)+" mins") 
                    dev.updateStateOnServer(key="minsPast7Days", value=mW3, uiValue=str(mW3)+" mins")
                    dev.updateStateOnServer(key="minsPast30Days", value=mM3, uiValue=str(mM3)+" mins")                                
                dev.setErrorStateOnServer(None)
                self.sleep(2)
            
                if dev.deviceTypeId == "tapoPlug":
                    response = tapoPlugApi.getDeviceEnergyUsage(device)
                    if self.logComms:
                        indigo.server.log("Energy usage for %s" % dev.name)
                        indigo.server.log(str(response))
                    jresponse = json.loads(response)
                    wattsString = str(jresponse["result"]["current_power"])
                    watts = round(float(wattsString)/1000, 1)
                    kwhd3 = int(str(jresponse["result"]["today_energy"]))
                    kwhd = round(kwhd3/1000, 1)
                    kwhm3 = int(str(jresponse["result"]["month_energy"]))
                    kwhm = round(kwhm3/1000, 1)
                    dev.updateStateOnServer(key="energyCurLevel", value=watts, uiValue=str(watts)+" W")
                    dev.updateStateOnServer(key="kWhDay", value=kwhd, uiValue=str(kwhd)+" kWh")
                    dev.updateStateOnServer(key="kWhMonth", value=kwhm, uiValue=str(watts)+" kWh")
                    if self.showEnergyInNotes:
                        dev.description = str(watts)+" W"
                        dev.replaceOnServer()
                    elif dev.description != "":
                        dev.description = ""
                        dev.replaceOnServer()
            except:
                dev.setErrorStateOnServer("Error")
                if self.logComms:
                    indigo.server.log("%s comms failed" % dev.name, isError=True)




    ########################################################
    # Validation methods for plugin prefs and device config
    ########################################################
    
    def validateDeviceConfigUi(self, valuesDict, typeId, devId):    

        errorMsgDict = indigo.Dict()                
        try:
            ip = ipaddress.ip_address(valuesDict["address"])
            return (True, valuesDict)
        except ValueError:
            errorMsgDict["address"] = "Invalid IP Address"                            
            return (False, valuesDict, errorMsgDict)
        



    ########################################
    # Smart plug/bulb action callback
    ########################################
    
    # Main switch action bottleneck called by Indigo Server.
    def actionControlDevice(self, action, dev):

        if action.deviceAction == indigo.kDeviceAction.TurnOn:
            device = {
                "tapoIp": dev.address,
                "tapoEmail": self.userName,
                "tapoPassword": self.userPass
            }
            try:
                response = tapoPlugApi.plugOn(device)
                dev.updateStateOnServer("onOffState", True)
                dev.updateStateImageOnServer(indigo.kStateImageSel.PowerOn)
            except:
                indigo.server.log("%s 'on' command failed" % dev.name)
                
        elif action.deviceAction == indigo.kDeviceAction.TurnOff:
            device = {
                "tapoIp": dev.address,
                "tapoEmail": self.userName,
                "tapoPassword": self.userPass
            }
            try:
                response = tapoPlugApi.plugOff(device)
                dev.updateStateOnServer("onOffState", False)
                dev.updateStateImageOnServer(indigo.kStateImageSel.PowerOff)
            except:
                indigo.server.log("%s 'off' command failed" % dev.name)
                
        elif action.deviceAction == indigo.kDeviceAction.SetBrightness:
            new_brightness = action.actionValue
            if new_brightness >0:
                device = {
                    "tapoIp": dev.address,
                    "tapoEmail": self.userName,
                    "tapoPassword": self.userPass
                }
                try:
                    response = tapoPlugApi.setBrightness(device, new_brightness)
                    dev.updateStateOnServer(key="brightnessLevel", value=new_brightness, uiValue=str(new_brightness)+"%")
                except:
                    indigo.server.log("%s brightness change command failed" % dev.name, isError=True)
            else:
                # Tapo will not accept brightness=0 so turn off lamp instead
                device = {
                    "tapoIp": dev.address,
                    "tapoEmail": self.userName,
                    "tapoPassword": self.userPass
                }
                try:
                    response = tapoPlugApi.plugOff(device)
                    dev.updateStateOnServer("onOffState", False)
                    dev.updateStateImageOnServer(indigo.kStateImageSel.PowerOff)
                except:
                    indigo.server.log("%s 'off' command failed" % dev.name)                
        else:
            indigo.server.log("%s action is not currently supported" % dev.name, isError=True)


